%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the Padus weaver, build 2009.09.15
% 
% REQUIREMENTS:
% - SWI-Prolog
% - The SWI-Prolog SGML/XML parser
% (Under Linux, install the 'swi-prolog' and 'swi-prolog-sgml' packages.  Under
% Windows, run the SWI-Prolog installer, making sure the 'SGML/XML/HTML parser'
% component is selected.)
% 
% USAGE:
% $ swipl -f padus/padus.pl -t 'weave("<input-directory>", "<deployment-file>", "<output-directory>")'
% 
% EXAMPLE:
% $ swipl -f padus/padus.pl -t 'weave("examples/", "Deployment.xml", "output/")'
% $ swipl -f padus/padus.pl -t 'weave("examples/", "Deployment2.xml", "output/")'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- consult('deployment').

% Configures the logging system to only show INFO messages
% See logging.pl for more information about the logging system
logging_level(Levels) :-
		Levels = ['INFO'].

% Weaves the composition specified by a certain deployment file
% Parameters:
% - InputDirString: A string that specifies where the deployment file and any
%                   aspect and process files are to be found.  For example:
%                   "examples/"
% - DeploymentFileString: A string that specifies the name of the deployment
%                         file.  For example: "Deployment.xml"
% - OutputDirString: A string that specifies where the woven process files
%                    should be placed.  For example: "output/"
weave(InputDirString, DeploymentFileString, OutputDirString) :-
		info('This is the Padus weaver, build 2009.08.13'),
		info(''),
		string_to_atom(InputDirString, InputDir),
		string_to_atom(DeploymentFileString, DeploymentFile),
		string_to_atom(OutputDirString, OutputDir),
		info(['Input dir       : ', InputDir]),
		info(['Deployment file : ', DeploymentFile]),
		info(['Output dir      : ', OutputDir]),
		info(''),
		!,
		clear,
		concat_atom([InputDir, DeploymentFile], DeploymentFilePath),
		loadAndParseFile(DeploymentFilePath, 'deployment'),
		!,
		findAspectNames(AspectNames),
		info(['Aspect names    : ', AspectNames]),
		findAspectPaths(InputDir, AspectNames, AspectPaths),
		info(['Aspect paths    : ', AspectPaths]),
		defineAspects(AspectNames, AspectPaths),
		info(''),
		!,
		findProcessNames(ProcessNames),
		info(['Process names   : ', ProcessNames]),
		findProcessPaths(InputDir, ProcessNames, ProcessPaths),
		info(['Process paths   : ', ProcessPaths]),
		defineProcesses(ProcessNames, ProcessPaths),
		info(''),
		!,
		setOutputDir(OutputDir),
		iterateOverProcesses,
		info('').

findAspectNames(AspectNames) :-
		findall(AspectName,
		        (child(0, AspectNodeId, _, 'deployment'),
		         node(AspectNodeId, AspectNodeName, 'deployment'),
		         split(AspectNodeName, _, ':', 'aspect'),
		         attribute(AspectNodeId, 'name', AspectName, 'deployment')),
		        AspectNames).

findAspectPaths(InputDir, [AspectName | AspectNames], [AspectPath | AspectPaths]) :-
		concat_atom([InputDir, AspectName, '.xml'], AspectPath),
		findAspectPaths(InputDir, AspectNames, AspectPaths).
findAspectPaths(InputDir, [], []).

findProcessNames(ProcessNames) :-
		findall(ProcessName,
		        (child(0, PrecedenceNodeId, _, 'deployment'),
		         node(PrecedenceNodeId, PrecedenceNodeName, 'deployment'),
		         split(PrecedenceNodeName, _, ':', 'precedence'),
		         attribute(PrecedenceNodeId, 'process', ProcessName, 'deployment')),
		        ProcessNames).

findProcessPaths(InputDir, [ProcessName | ProcessNames], [ProcessPath | ProcessPaths]) :-
		concat_atom([InputDir, ProcessName, '.bpel'], ProcessPath),
		findProcessPaths(InputDir, ProcessNames, ProcessPaths).
findProcessPaths(InputDir, [], []).
